using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using System.Resources;
using System.Collections;

namespace convertor
{
    public partial class ConvertorForm : Form
    {
        const int MAX_FILE_SIZE = 60000;
        private OpenFileDialog ofd;
        private SaveFileDialog sfd;
        private String installDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName);
        About aboutDlg;
        class Ebook
        {
            public String inputFileName;
            public String tempInputFileName;
            public String title;
            public String pictureFileName;
            public String outputFileName;
            public bool forTI89;
            public bool forTI92;
        }
        private List<Ebook> ebooks;

        public ConvertorForm()
        {
            InitializeComponent();
            ofd = new OpenFileDialog();
            sfd = new SaveFileDialog();
            ebooks = new List<Ebook>();
            aboutDlg = new About();
            toolTip1.SetToolTip(inputFileLabel, "The file to be converted to ebook");
            toolTip1.SetToolTip(inputFileBox, "The file to be converted to ebook");
            toolTip1.SetToolTip(inputFileButton, "Browse...");
            toolTip1.SetToolTip(outputFileLabel, "The converted ebook file");
            toolTip1.SetToolTip(outputFileBox, "The converted ebook file");
            toolTip1.SetToolTip(outputFileButton, "Browse...");
            toolTip1.SetToolTip(titleLabel, "The title for the ebook");
            toolTip1.SetToolTip(titleBox, "The title for the ebook");
            toolTip1.SetToolTip(pictureLabel, "The binary image used as the cover of the ebook (created with iStudio)");
            toolTip1.SetToolTip(pictureBox, "The binary image used as the cover of the ebook (created with iStudio)");
            toolTip1.SetToolTip(pictureButton, "Browse...");
            toolTip1.SetToolTip(addToListButton, "Add to list >>");
            toolTip1.SetToolTip(RemoveFromListButton, "Remove from list");
            toolTip1.SetToolTip(simpleConvertButton, "Convert only this one");
            toolTip1.SetToolTip(convertAllButton, "Convert all in the list");
            toolTip1.SetToolTip(ti89CheckBox, "Create for TI-89");
            toolTip1.SetToolTip(ti92CheckBox, "Create for TI-92 Plus");
        }

        private void checkInput()
        {
            if (actualTextBox.Text.Trim() != "" || inputFileBox.Text.Trim() != "")
            {
                addToListButton.Enabled = true;
                simpleConvertButton.Enabled = true;
            }
            else
            {
                addToListButton.Enabled = false;
                simpleConvertButton.Enabled = false;
            }
        }

        private void inputFileBox_TextChanged(object sender, EventArgs e)
        {
            if (inputFileBox.Text.Trim() != "")
                actualTextBox.Text = "";
            checkInput();
        }

        private void actualTextBox_TextChanged(object sender, EventArgs e)
        {
            if (actualTextBox.Text.Trim() != "")
                inputFileBox.Text = "";
            checkInput();
        }

        private void inputFileButton_Click(object sender, EventArgs e)
        {
            ofd.Reset();
            ofd.Filter = "text files (*.txt)|*.txt|All files (*.*)|*.*";
            ofd.FilterIndex = 1;
//            openFileDialog1.RestoreDirectory = true;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
//                if ((myStream = openFileDialog1.OpenFile()) != null)
//                {
//                    myStream.Close();
//                }
                inputFileBox.Text = ofd.FileName;
                titleBox.Text = Path.GetFileNameWithoutExtension(inputFileBox.Text);
                String outfilename = Path.GetFileNameWithoutExtension(inputFileBox.Text);
                outfilename = outfilename.Replace(' ', '_');
                if (outfilename.Length > 8)
                    outfilename = outfilename.Substring(0, 8);
                outputFileBox.Text = Path.GetDirectoryName(inputFileBox.Text) + "\\" + outfilename;
            }

        }

        private void pictureButton_Click(object sender, EventArgs e)
        {
            ofd.Reset();
            ofd.Filter = "Binary files (*.bin)|*.bin";
            ofd.FilterIndex = 1;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                pictureBox.Text = ofd.FileName;
            }
        }

        private void outputFileButton_Click(object sender, EventArgs e)
        {
            sfd.Reset();
            sfd.Filter = "TI-89 files (*.89y)|*.89y";
            sfd.FilterIndex = 1;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                outputFileBox.Text = sfd.FileName;
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (!ti89CheckBox.Checked)
                ti92CheckBox.Checked = true;
        }

        private void ti92CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (!ti92CheckBox.Checked)
                ti89CheckBox.Checked = true;
        }

        private Ebook createEbookObject()
        {
            Ebook ebook = new Ebook();
            if (inputFileBox.Text.Trim() != "")
                ebook.inputFileName = inputFileBox.Text.Trim();
            else if (actualTextBox.Text.Trim() != "")
            {
                ebook.tempInputFileName = Path.GetTempFileName();
                using (FileStream fs = File.Open(ebook.tempInputFileName, FileMode.Open, FileAccess.Write, FileShare.None))
                {
                    Byte[] info = new UTF8Encoding(true).GetBytes(actualTextBox.Text);
                    // Add some information to the file.
                    fs.Write(info, 0, info.Length);
                }
            }
            else 
            {
                MessageBox.Show(this, "You must specify an input file or place some text in the box.", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                inputFileBox.Focus();
                return null;
            }
            if (outputFileBox.Text.Trim() != "")
                ebook.outputFileName = outputFileBox.Text.Trim();
            else if (ebook.inputFileName != null)
                ebook.outputFileName = ebook.inputFileName;
            else
            {
                MessageBox.Show(this, "You must specify output file name.", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                outputFileBox.Focus();
                return null;
            }

            String outfilename = Path.GetFileNameWithoutExtension(ebook.outputFileName);
            outfilename = outfilename.Replace(' ', '_');
            if (outfilename.Length > 8)
                outfilename = outfilename.Substring(0, 8);
            ebook.outputFileName = Path.GetDirectoryName(ebook.outputFileName) + "\\" + outfilename;

            if (titleBox.Text.Trim() != "")
                ebook.title = titleBox.Text.Trim();
            else if (inputFileBox.Text.Trim() != "")
            {
                ebook.title = Path.GetFileNameWithoutExtension(ebook.inputFileName);
                if (ebook.title.Length > 18)
                    ebook.title = ebook.title.Substring(0, 18);
            }
            else
            {
                ebook.title = "No Title";
            }

            if (pictureBox.Text.Trim() != "")
                ebook.pictureFileName = pictureBox.Text.Trim();
            else
                ebook.pictureFileName = installDir + "\\defaultpic.bin";

            ebook.forTI89 = ti89CheckBox.Checked;
            ebook.forTI92 = ti92CheckBox.Checked;
            inputFileBox.ResetText();
            outputFileBox.ResetText();
            titleBox.ResetText();
            pictureBox.ResetText();
            actualTextBox.ResetText();
            return ebook;
        }

        private void addToListButton_Click(object sender, EventArgs e)
        {
            Ebook ebook = createEbookObject();
            if (ebook != null)
            {
                ebooks.Add(ebook);
                String itemText = "";
                if (ebook.inputFileName != null)
                    itemText += ebook.inputFileName;
                else if (ebook.title != null)
                    itemText += ebook.title;
                else
                    itemText += ebooks.Count;
                itemText += "  ->  " + ebook.outputFileName;
                fileItemsListBox.Items.Add(itemText);
                disableButtons();
            }
        }

        private void simpleConvertButton_Click(object sender, EventArgs e)
        {
            Ebook ebook = createEbookObject();
            if (ebook != null)
                convertEbook(ebook);
        }

        private void convertEbook(Ebook ebook)
        {
            string tmpFile = Path.GetTempFileName();
            if (ebook.inputFileName != null)
                Convertor.convert(ebook.inputFileName, tmpFile);
            else
                Convertor.convert(ebook.tempInputFileName, tmpFile);


            FileStream tmpfs = new FileStream(tmpFile, FileMode.Open);
            long FileSize = tmpfs.Length;
            String[] inputFiles;
            if (FileSize > MAX_FILE_SIZE)
            {
                inputFiles = new String[(int)Math.Ceiling(FileSize*1.0 / MAX_FILE_SIZE)];
                byte[] info = new byte[MAX_FILE_SIZE];
                for (int i = 0; i < inputFiles.Length; i++)
                {
                    inputFiles[i] = tmpFile + i;
                    using (FileStream fs = new FileStream(inputFiles[i], FileMode.CreateNew))
                    {
                        int cnt = tmpfs.Read(info, 0, (int)Math.Min(FileSize - MAX_FILE_SIZE * i, MAX_FILE_SIZE));
                        // Add some information to the file.
                        fs.Write(info, 0, cnt);
                        fs.Close();
                    }
                }
            }
            else
            {
                inputFiles = new String[] {tmpFile};
            }
            tmpfs.Close();

            String program = installDir + "\\ttebkgen.exe";

            String tmpPic = Path.GetTempFileName();
            File.Copy(ebook.pictureFileName, tmpPic, true);
            String currentDirectory = Environment.CurrentDirectory;
            Environment.CurrentDirectory = Path.GetDirectoryName(tmpPic);

            for (int i = 0; i < inputFiles.Length; i++)
            {
                String args = "";
                args += " -t " + Path.GetFileName(tmpPic);
                if (ebook.forTI89 && !ebook.forTI92)
                    args += " -89";
                if (ebook.forTI92 && !ebook.forTI89)
                    args += " -92";
                args += " \"" + inputFiles[i] + "\"";
                String varname = Path.GetFileNameWithoutExtension(ebook.outputFileName);
                if (varname.Length > 8)
                    varname = varname.Substring(0, 8);

                String title = ebook.title;
                if (inputFiles.Length > 1)
                {
                    if (varname.Length > 7 - inputFiles.Length / 10)
                        varname = varname.Substring(0, 7 - inputFiles.Length / 10);
                    varname += i;
                    if (title.Length > 17 - inputFiles.Length / 10)
                        title = title.Substring(0, 17 - inputFiles.Length / 10);
                    title += i;
                }
                args += " \"" + varname + "\"";
                args += " \"" + title + "\"";

                //run ttebkgen.exe
                ProcessStartInfo psi = new ProcessStartInfo(program, args);
                psi.UseShellExecute = false;
                psi.CreateNoWindow = true;
                psi.RedirectStandardOutput = false;
                psi.RedirectStandardError = false;
                Process ttebkgen = Process.Start(psi);
                ttebkgen.WaitForExit();
                if (ttebkgen.ExitCode != 0)
                    MessageBox.Show(this, "There was a problem in converting ebooks.", "Error",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                if (ebook.forTI89 && File.Exists(varname + ".89y"))
                {
                    File.Copy(varname + ".89y", Path.GetDirectoryName(ebook.outputFileName) + "\\" + varname + ".89y", true);
                    File.Delete(varname + ".89y");
                }
                if (ebook.forTI92 && File.Exists(varname + ".9xy"))
                {
                    File.Copy(varname + ".9xy", Path.GetDirectoryName(ebook.outputFileName) + "\\" + varname + ".9xy", true);
                    File.Delete(varname + ".9xy");
                }
                File.Delete(inputFiles[i]);
            }


            if (inputFiles.Length > 1)
                File.Delete(tmpFile);
            File.Delete(tmpPic);
            if (ebook.tempInputFileName != null)
                File.Delete(ebook.tempInputFileName);
            Environment.CurrentDirectory = currentDirectory;
        }

        private void RemoveFromListButton_Click(object sender, EventArgs e)
        {
            ListBox.SelectedIndexCollection selectedIndices = fileItemsListBox.SelectedIndices;
            for (int i = selectedIndices.Count - 1; i >= 0; i--)
            {
                int index = selectedIndices[i];
                fileItemsListBox.Items.RemoveAt(index);
                ebooks.RemoveAt(index);
            }
            disableButtons();
        }

        private void convertAllButton_Click(object sender, EventArgs e)
        {
            for (int i = fileItemsListBox.Items.Count - 1; i >= 0; i--)
            {
                convertEbook(ebooks[i]);
                fileItemsListBox.Items.RemoveAt(i);
                ebooks.RemoveAt(i);
            }
            disableButtons();
        }

        private void disableButtons()
        {
            bool enabled = fileItemsListBox.Items.Count > 0;
            RemoveFromListButton.Enabled = enabled;
            convertAllButton.Enabled = enabled;
        }

        private void helpButton_Click(object sender, EventArgs e)
        {
            Process pdfOpen = new Process();
            pdfOpen.StartInfo.FileName = installDir + "\\Help.pdf";
            pdfOpen.StartInfo.CreateNoWindow = true;
            pdfOpen.Start();
        }

        private void infoButton_Click(object sender, EventArgs e)
        {
            aboutDlg.ShowDialog();
        }

        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(actualTextBox.SelectedText);
            actualTextBox.SelectedText = "";
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(actualTextBox.SelectedText);
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            actualTextBox.SelectedText = Clipboard.GetText();
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            actualTextBox.SelectedText = "";
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            actualTextBox.SelectAll();
        }

        private void actualTextBox_SelectionChanged(object sender, EventArgs e)
        {
            cutToolStripMenuItem.Enabled =
                copyToolStripMenuItem.Enabled =
                deleteToolStripMenuItem.Enabled = actualTextBox.SelectedText != null &&
                actualTextBox.SelectedText != "";
        }
    }
}